const path = require('path');
const fs = require('fs');
const SVGDoc = require('svg-sprite-loader/lib/svg-document');

const TEMP_FOLDER = path.resolve(__dirname, '../.temp');
if (!fs.existsSync(TEMP_FOLDER)) {
  fs.mkdirSync(TEMP_FOLDER);
}

const process = function(source) {
  if (source.indexOf('.embed(') > -1) {
    const reg = /.embed\((.|\n)*?\)/g;
    const matches = source.match(reg);
    const icons = [];

    matches.forEach((match) => {
      const iconString = match.replace(/\s|\[|\]/g, '').replace('.embed(', '').replace(')', '');
      if (iconString.startsWith('\'')) {
        icons.push(...iconString.split(','));
      }
    });

    const scripts = [];
    if (icons.length > 0) {
      scripts.push('\nvar __vue_lightning_sprite__ = require(\'svg-sprite-loader/lib/web/global-sprite.js\');');
      icons.forEach((icon) => {
        if (icon) {
          const names = icon.replace(/'/g, '').split('.');
          const category = names[0];
          const name = names[1];
          const filePath = path.resolve(__dirname, `../../@salesforce-ux/design-system/assets/icons/${category}/${name}.svg`);
          const data = fs.readFileSync(filePath);
          const doc = new SVGDoc(data);
          doc.$svg.attr('id', `svg__${category}__${name}`);
          const content = doc.toString(SVGDoc.OUTPUT_FORMAT.SYMBOL).replace(' fill="#fff"', '');
          const svgModule = [
            'module.exports = \'',
            content,
            '\';\n'
          ];

          const tempFile = path.join(TEMP_FOLDER, `${name}.js`);
          fs.writeFileSync(tempFile, svgModule.join(''));

          scripts.push(`\n__vue_lightning_sprite__.add(require('${tempFile}'), 'symbols');`);
        }
      });
    }

    return scripts.join('');
  }

  return '';
};

module.exports = function(source) {
  const ext = path.extname(this.resourcePath).toLowerCase();
  let content = '';
  if (ext === '.vue') {
    content = fs.readFileSync(this.resourcePath).toString();
  } else if (ext === '.js') {
    content = source;
  }

  const script = process(content);

  return `${script}\n\n${source}`;
};
